/*version du 10 juin 2021 : 

      Projet 4A Fusée à eau, Clément, Anthony et Numa
      Nouveautés : ajout d'un capteur de pression et de d'avertisseurs sonores et lumineuses 
      
      Un appui sur le Declencheur (télécommande) ouvre le diaphrame un second appui est nécessaire pour le refermer
      Le capteur de pression indique la pression de l'eau dans la fusée. Une fois la pression limite atteinte (2 BARS), une alarme et un gyrophare se déclanchent. 
      Le décollage est imminent.  
      
*/

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DECLARATIONS CONSTANTES ---------------------------------------------------------------------------------------//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

//Bibliothèques
#include <Servo.h>        //appel la bibliothèque servo
#include <Wire.h>        //appel la bibliothèque wire
Servo servo;

//Servomoteur - constantes

int position_diaphragme_ouvert = 40;  // Position du servomoteur pour diaphrame ouvert
int position_diaphragme_ferme  = 160;   // Position du servomoteur pour diaphrame fermé

//Ports 
#define DeclencheurPin A0       // Pin déclencheur
#define flashPin 9              // Pin flash
#define sirenePin 6             // Pin sirène
#define captPressionPin A7      // Pin Capteur
#define servoPin 3              // Pin servos
#define electroVannePin 12      // Pin electrovanne
#define debitmetrePin A1        // Pin débitmètre
#define anenometrePin A2        // Pin anémomètre

//constantes
const int seuil_pression = 2;

//Etats - variables 
int etatDeclencheur = 0;    
int dernierEtatDeclencheur  = 0;     
int positionDiaphragme = 1;
int electroVanne = 1;
   
//Variables
float pression;   //Pression en fonction de la sortie du capteur de pression


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// SETUP - INITIALISATIONS ---------------------------------------------------------------------------------------//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void setup() 
{
  Serial.begin(9600);               // ouverture du port serial, fréquence 9600 bps
  pinMode(DeclencheurPin, INPUT); 
  pinMode(flashPin, OUTPUT); 
  pinMode(sirenePin, OUTPUT);
  pinMode(electroVannePin, OUTPUT);
  servo.attach(servoPin); 

  //test diaphragme 

  servo.write(position_diaphragme_ouvert);   //mise à zéro du diaphragme (ouvert)
  delay(1000);
  servo.write(position_diaphragme_ferme);   // fermture du diaphragme pour test
  delay(1000);
  servo.write(position_diaphragme_ouvert);  // réouverture du diaphragme
  delay(1000);

  //test flash
  
  digitalWrite(flashPin, LOW);  // mise à zero de la lampe flash
  digitalWrite(flashPin, HIGH);  // allumage de la lampe
  delay(2500);
  digitalWrite(flashPin, LOW);  // extinction de la lampe après 2,5 seconde
  
  //test sirène

  digitalWrite(sirenePin, LOW);  //mise à zero de la sirène
  digitalWrite(sirenePin, HIGH);  //allumage sirène
  delay(150);
  digitalWrite(sirenePin, LOW);  // extinction sièrne après 50 ms

  //test electro vanne

  //digitalWrite(electroVannePin, HIGH) ;
  //digitalWrite(electroVannePin, LOW) ;
  
}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// LOOP - BOUCLE -------------------------------------------------------------------------------------------------//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void loop() 
  {
  
  Capter_pression();                                  // Appel de la fonction de récupération de la pression

  etatDeclencheur = digitalRead(DeclencheurPin);      //Lecture de l'état du déclancheur

  if (etatDeclencheur != dernierEtatDeclencheur)
  {
    if (etatDeclencheur) //si etatDeclencheur est différent de 1
    { 
      if (positionDiaphragme) // Si Led allumée
      {
        positionDiaphragme = 0;
        servo.write(position_diaphragme_ferme);          // Ouverture du diaphragme
      }
      else 
      {
        positionDiaphragme = 1;
        servo.write(position_diaphragme_ouvert);          // Ouverture du diaphragme 
      }
    }
    
    dernierEtatDeclencheur = etatDeclencheur;
  
  }

  delay(50);

}


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FONCTIONS ANNEXES ---------------------------------------------------------------------------------------------//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


void Capter_pression()   // Fonction dédiée à la capture de la valeur de la pression dans le système
{   
  float V = analogRead(captPressionPin) * 5.00 / 1024;
  float pression =  2.588*V - 1.189;
  //float pression =  (V - offSet_Pression ) * 2.588
  
  Serial.print("tension :");
  Serial.print(V, 3);
  Serial.print("  V");

  Serial.print(" Pression:");
  Serial.print(pression, 1);
  Serial.println("  bar");
  Serial.println();
  
  if (pression >= seuil_pression)     // Si la pression est supérieure au seuil  /!\ DANGER, ecartez vous !
    {
      digitalWrite(flashPin, HIGH);   //allumage flash     
      digitalWrite(sirenePin, HIGH) ; //allumage sirène
    }
    
   else                               // Si la pression n'est pas suffisante
    {
      digitalWrite(flashPin, LOW);            
      digitalWrite(sirenePin, LOW); 
    }
        
}
